from .plctopology cimport pd_code_t, bool

cdef class PlanarDiagram

cdef class _OwnedObjectList:
    cdef PlanarDiagram parent
    cdef object objs
    cdef object _new_child_object(self, long i)

cdef class _EdgeList(_OwnedObjectList):
    pass
cdef class _CrossingList(_OwnedObjectList):
    pass
cdef class _ComponentList(_OwnedObjectList):
    pass
cdef class _FaceList(_OwnedObjectList):
    pass

cdef class PlanarDiagram:
    """Class which represents a PD code (planar diagram). There are
    a few different ways to create ``PlanarDiagram``\ s:

    The constructor will create a new empty diagram with `max_verts`
    blank crossings.

    There are two class methods for reading diagrams from files, both
    from files written in ``PlanarDiagram``'s own format, and from files
    written in Mathematica's KnotTheory format.

    There are multiple class methods which create new ``PlanarDiagram``\ s
    of basic examples; their docstrings contain some specific information."""
    cdef pd_code_t *p
    cdef readonly _EdgeList edges
    """A sequence of :py:class:`Edge` which belong to this diagram"""
    cdef readonly _CrossingList crossings
    """A sequence of :py:class:`Crossing` which belong to this diagram"""
    cdef readonly _ComponentList components
    """A sequence of :py:class:`Component` which belong to this diagram"""
    cdef readonly _FaceList faces
    """A sequence of :py:class:`Face` which belong to this diagram"""

    cdef readonly bool thin
    cdef readonly bool hashed
    cdef char* _homfly

    cdef public int equiv_type

    cdef regenerate_py_os(self)

cdef PlanarDiagram PlanarDiagram_wrap(pd_code_t *pd, bool thin=*)

cdef public api:
    pd_code_t **pd_simplify(pd_code_t *pd, int *ndias)
